using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;

namespace Expl.Auxiliary {
   public static class Enumerable {
      public static IEnumerable<int> Counter(int Start) {
         for (int i = Start; ; i++)
            yield return i;
      }

      public static IEnumerable<int> Counter(int Start, int Step) {
         if (Step == 0) yield break;
         for (int i = Start; ; i += Step)
            yield return i;
      }

      public static string Join(IEnumerable List, string Delimiter) {
         StringBuilder sb = new StringBuilder();
         bool IterateFlag = false;

         foreach (var o in List) {
            if (IterateFlag) sb.Append(Delimiter);
            else IterateFlag = true;
            sb.Append(o.ToString());
         }

         return sb.ToString();
      }

      public static SpyEnumerator<T> GetSpyEnumerator<T>(IEnumerator<T> Iterator) {
         return new SpyEnumerator<T>(Iterator);
      }

      public static SpyEnumerator<T> GetSpyEnumerator<T>(IEnumerable<T> Iterable) {
         return new SpyEnumerator<T>(Iterable.GetEnumerator());
      }

      public static SpyEnumerator GetSpyEnumerator(IEnumerator Iterator) {
         return new SpyEnumerator(Iterator);
      }

      public static SpyEnumerator GetSpyEnumerator(IEnumerable Iterable) {
         return new SpyEnumerator(Iterable.GetEnumerator());
      }
   }

   public class SpyEnumerator<T> : IEnumerator<T> {
      IEnumerator<T> _Iterator;
      T _Current, _Peek;
      bool _HasData = false;
      bool _HasMore = false;

      public SpyEnumerator(IEnumerator<T> Iterator) {
         _Iterator = Iterator;
         if (_HasMore = _Iterator.MoveNext()) _Peek = _Iterator.Current;
      }

      public void Dispose() {
         _Iterator = null;
      }

      public bool HasMore { get { return _HasMore; } }

      public T Peek {
         get {
            if (_HasMore) return _Peek;
            else throw new InvalidOperationException();
         }
      }

      object IEnumerator.Current { get { return _Current; } }

      public T Current {
         get {
            if (_HasData) return _Current;
            else throw new InvalidOperationException();
         }
      }

      public bool MoveNext() {
         if (_HasMore) {
            _Current = _Peek;
            _HasData = true;
            if (_HasMore = _Iterator.MoveNext()) _Peek = _Iterator.Current;
         }
         else {
            _HasData = false;
         }
         return _HasData;
      }

      public void Reset() {
         _Iterator.Reset();
         _HasMore = _Iterator.MoveNext();
         if (_HasMore) _Peek = _Iterator.Current;
      }
   }

   public class SpyEnumerator : IEnumerator {
      IEnumerator _Iterator;
      object _Current, _Peek;
      bool _HasData = false;
      bool _HasMore = false;

      public SpyEnumerator(IEnumerator Iterator) {
         _Iterator = Iterator;
         if (_HasMore = _Iterator.MoveNext()) _Peek = _Iterator.Current;
      }

      public void Dispose() {
         _Iterator = null;
      }

      public bool HasMore { get { return _HasMore; } }

      public object Peek {
         get {
            if (_HasMore) return _Peek;
            else throw new InvalidOperationException();
         }
      }

      public object Current {
         get {
            if (_HasData) return _Current;
            else throw new InvalidOperationException();
         }
      }

      public bool MoveNext() {
         if (_HasMore) {
            _Current = _Peek;
            _HasData = true;
            if (_HasMore = _Iterator.MoveNext()) _Peek = _Iterator.Current;
         }
         else {
            _HasData = false;
         }
         return _HasData;
      }

      public void Reset() {
         _Iterator.Reset();
         _HasMore = _Iterator.MoveNext();
         if (_HasMore) _Peek = _Iterator.Current;
      }
   }
}