using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;

namespace Expl.Itinerary {

   /// <summary>
   /// Intersecting schedules collection of two ISchedule objects where event A intersects with event B
   /// </summary>
   [Description("Boolean Intersection")]
   public class BoolIntersectionSchedule : ICompositeSchedule {
      protected ISchedule _ScheduleA, _ScheduleB;

      /// <summary>
      /// Default contructor
      /// </summary>
      public BoolIntersectionSchedule()
         : this(new VoidSchedule(), new VoidSchedule()) { }

      /// <summary>
      /// Constructor
      /// </summary>
      /// <param name="ScheduleA">Schedule A</param>
      /// <param name="ScheduleB">Schedule B</param>
      public BoolIntersectionSchedule(ISchedule ScheduleA, ISchedule ScheduleB) {
         _ScheduleA = ScheduleA;
         _ScheduleB = ScheduleB;
      }

      /// <summary>
      /// Constructor for schedule list
      /// <remarks>Will only enumerate the first two items from the list</remarks>
      /// </summary>
      /// <param name="List">Enumerable list of ISchedule objects</param>
      public BoolIntersectionSchedule(IEnumerable<ISchedule> List) {
         var Iter = List.GetEnumerator();
         if (Iter.MoveNext()) {
            _ScheduleA = Iter.Current;
            if (Iter.MoveNext())
               _ScheduleB = Iter.Current;
            else
               _ScheduleB = new VoidSchedule();
         }
         else {
            _ScheduleA = _ScheduleB = new VoidSchedule();
         }
      }

      public void Dispose() {
         _ScheduleA = null;
         _ScheduleB = null;
      }

      public override string ToString() {
         return string.Format("({0} && {1})", _ScheduleA.ToString(), _ScheduleB.ToString());
      }

      public IEnumerable<ISchedule> Schedules {
         get {
            yield return _ScheduleA;
            yield return _ScheduleB;
         }
      }

      public int MinSchedules {
         get { return 2; }
      }

      public int MaxSchedules {
         get { return 2; }
      }

      // Optimized 2008-07-13
      public IEnumerable<TimedEvent> GetRange(DateTime RangeStart, DateTime RangeEnd) {
         yield break;
      }
   }
}
