using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Text;
using Expl.Auxiliary;

namespace Expl.Itinerary {

   /// <summary>
   /// Difference
   /// Non-intersecting schedules collection of two or more ISchedule objects where event times do not overlap.
   /// (****[..)****]
   /// Generate difference of two datetime spans
   /// </summary>
   [Description("Difference")]
   public class DifferenceSchedule : ICompositeSchedule {
      protected ListSchedule _Schedule;

      /// <summary>
      /// Default constructor for empty list
      /// </summary>
      public DifferenceSchedule()
         : this(new VoidSchedule(), new VoidSchedule()) { }

      /// <summary>
      /// Constructor for two schedules
      /// </summary>
      /// <param name="ScheduleA">Schedule A</param>
      /// <param name="ScheduleB">Schedule B</param>
      public DifferenceSchedule(ISchedule ScheduleA, ISchedule ScheduleB) {
         _Schedule = new ListSchedule(ScheduleA, ScheduleB);
      }

      /// <summary>
      /// Constructor for many schedules
      /// </summary>
      /// <param name="List">Enumerable list of ISchedule objects</param>
      public DifferenceSchedule(IEnumerable<ISchedule> List) {
         _Schedule = new ListSchedule(List);
      }

      public void Dispose() {
         _Schedule = null;
      }

      public IEnumerable<ISchedule> Schedules {
         get { return _Schedule.Schedules; }
      }

      public int MinSchedules {
         get { return 2; }
      }

      public int MaxSchedules {
         get { return int.MaxValue; }
      }

      public override string ToString() {
         StringBuilder sb = new StringBuilder();
         sb.Append("(");
         sb.Append(Enumerable.Join(Schedules, " !& "));
         sb.Append(")");
         return sb.ToString();
      }

      // Optimized 2008-07-13
      public IEnumerable<TimedEvent> GetRange(DateTime RangeStart, DateTime RangeEnd) {
         yield break;
      }
   }
}
