﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;

namespace Expl.Itinerary {

   /// <summary>
   /// Intersecting schedules collection of two ISchedule objects where event times overlap.
   /// This differs with IntersectionSchedule in that there is no comparison between schedules.  All events are flattened to intersecting events.
   /// </summary>
   [Description("IntersectionAll")]
   public class IntersectionAllSchedule : ICompositeSchedule {
      protected ISchedule _Schedule;

      /// <summary>
      /// Constructor
      /// </summary>
      /// <param name="Schedule">Schedule subject</param>
      public IntersectionAllSchedule(ISchedule Schedule) {
         _Schedule = Schedule;
      }

      public void Dispose() {
         _Schedule = null;
      }

      public override string ToString() {
         return string.Format("&[{0}]", _Schedule.ToString());
      }

      public IEnumerable<ISchedule> Schedules {
         get { yield return _Schedule; }
      }

      public int MinSchedules {
         get { return 1; }
      }

      public int MaxSchedules {
         get { return 1; }
      }

      // Optimized 2008-07-16
      public IEnumerable<TimedEvent> GetRange(DateTime RangeStart, DateTime RangeEnd) {
         yield break;
      }
   }
}
