using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using Expl.Auxiliary;

namespace Expl.Itinerary {

   /// <summary>
   /// Union collection of ISchedule objects
   /// </summary>
   [Description("List")]
   public class ListSchedule : ICompositeSchedule {
      protected List<ISchedule> _Schedules;

      /// <summary>
      /// Constructor for empty list
      /// </summary>
      public ListSchedule() {
         _Schedules = new List<ISchedule>();
      }

      /// <summary>
      /// Constructor for single element in list
      /// </summary>
      /// <param name="A">Schedule</param>
      public ListSchedule(ISchedule A) {
         _Schedules = new List<ISchedule>();
         _Schedules.Add(A);
      }

      /// <summary>
      /// Constructor for two elements in list
      /// </summary>
      /// <param name="A">Schedule A</param>
      /// <param name="B">Schedule B</param>
      public ListSchedule(ISchedule A, ISchedule B) {
         _Schedules = new List<ISchedule>();
         _Schedules.Add(A);
         _Schedules.Add(B);
      }

      /// <summary>
      /// Constructor for many elements in list
      /// </summary>
      /// <param name="List">Enumerable list of ISchedule objects</param>
      public ListSchedule(IEnumerable<ISchedule> List) {
         _Schedules = new List<ISchedule>(List);
      }

      public void Dispose() {
         _Schedules.Clear();
         _Schedules = null;
      }

      public override string ToString() {
         StringBuilder sb = new StringBuilder();
         sb.Append("(");
         sb.Append(Enumerable.Join(_Schedules, ", "));
         sb.Append(")");
         return sb.ToString();
      }

      /// <summary>
      /// Get ISchedules collection
      /// </summary>
      public IEnumerable<ISchedule> Schedules {
         get { return _Schedules; }
      }

      public int MinSchedules {
         get { return 0; }
      }

      public int MaxSchedules {
         get { return int.MaxValue; }
      }

      const int MaxScheduleBufferSize = 20;

      // Optimized 2008-07-07
      public IEnumerable<TimedEvent> GetRange(DateTime RangeStart, DateTime RangeEnd) {
         yield break;
      }
   }
}
