using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using Expl.Auxiliary;

namespace Expl.Itinerary {

   /// <summary>
   /// Union collection of ISchedule objects
   /// </summary>
   [Description("Union")]
   public class UnionSchedule : ICompositeSchedule {
      protected ListSchedule _Schedule;

      /// <summary>
      /// Constructor for empty list
      /// </summary>
      public UnionSchedule() {
         _Schedule = new ListSchedule();
      }

      /// <summary>
      /// Constructor for single schedule
      /// </summary>
      /// <param name="A">Schedule</param>
      public UnionSchedule(ISchedule A) {
         _Schedule = new ListSchedule(A);
      }

      /// <summary>
      /// Constructor for two schedules
      /// </summary>
      /// <param name="A">Schedule A</param>
      /// <param name="B">Schedule B</param>
      public UnionSchedule(ISchedule A, ISchedule B) {
         _Schedule = new ListSchedule(A, B);
      }

      /// <summary>
      /// Constructor for many schedules
      /// </summary>
      /// <param name="List"></param>
      public UnionSchedule(IEnumerable<ISchedule> List) {
         _Schedule = new ListSchedule(List);
      }

      public void Dispose() {
         _Schedule = null;
      }

      public IEnumerable<ISchedule> Schedules {
         get { return _Schedule.Schedules; }
      }

      public int MinSchedules {
         get { return 1; }
      }

      public int MaxSchedules {
         get { return int.MaxValue; }
      }

      public override string ToString() {
         StringBuilder sb = new StringBuilder();
         sb.Append("(");
         sb.Append(Enumerable.Join(Schedules, " | "));
         sb.Append(")");
         return sb.ToString();
      }

      // Optimized 2008-07-13
      public IEnumerable<TimedEvent> GetRange(DateTime RangeStart, DateTime RangeEnd) {
         yield break;
      }
   }
}
