using System;
using System.Collections.Generic;
using System.Text;

namespace Expl.Itinerary {
   /// <summary>
   /// Interface for Schedule classes containing time schedule and delegate reaction.
   /// </summary>
   public interface ISchedule : IDisposable {
      ///// <summary>
      ///// Get user defined scheduld name
      ///// </summary>
      //string Name {
      //   get;
      //}

      /// <summary>
      ///  Return TDL syntax of schedule definition
      /// </summary>
      string ToString();

      /// <summary>
      /// Get events that intersect a range of time
      /// </summary>
      /// <param name="RangeStart">Start time</param>
      /// <param name="RangeEnd">End time</param>
      /// <returns>Enumeration of events in chronological order</returns>
      IEnumerable<TimedEvent> GetRange(DateTime RangeStart, DateTime RangeEnd);
   }

   public interface IPrimitiveSchedule : ISchedule { }

   public interface IFilterSchedule : ISchedule {
      /// <summary>
      /// Get associated base schedule
      /// </summary>
      ISchedule BaseSchedule { get; }
   }

   public interface ICompositeSchedule : ISchedule {
      /// <summary>
      /// List of associated schedules
      /// </summary>
      IEnumerable<ISchedule> Schedules { get; }

      /// <summary>
      /// Minimum number of associated schedules
      /// </summary>
      int MinSchedules { get; }

      /// <summary>
      /// Maximum number of associated schedules
      /// </summary>
      int MaxSchedules { get; }
   }
}
