using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Text.RegularExpressions;
using Expl.Auxiliary;

namespace Expl.Itinerary {

   public class CronSchedule : IPrimitiveSchedule {
      protected CronField _MinuteLookup;
      protected CronField _HourLookup;
      protected CronField _DayLookup;
      protected CronField _MonthLookup;
      protected CronField _DayOfWeekLookup;
      protected TimeSpan _Duration;

      /// <summary>
      /// Default constructor
      /// </summary>
      public CronSchedule()
         : this("*", "*", "*", "*", "*", TimeSpan.Zero) { }

      /// <summary>
      /// Constructor for cron specification of event intervals
      /// </summary>
      /// <param name="MinuteSpec">Minute spec</param>
      /// <param name="HourSpec">Hour spec</param>
      /// <param name="DaySpec">Day spec</param>
      /// <param name="MonthSpec">Month spec</param>
      /// <param name="DayOfWeekSpec">Day of week spec</param>
      /// <param name="Duration">Duration of event</param>
      public CronSchedule(string MinuteSpec, string HourSpec, string DaySpec, string MonthSpec, string DayOfWeekSpec, TimeSpan Duration) {
         _MinuteLookup = new CronField(MinuteSpec, 0, 59);
         _HourLookup = new CronField(HourSpec, 0, 23);
         _DayLookup = new CronField(DaySpec, 1, 31);
         _MonthLookup = new CronField(MonthSpec, 1, 12);
         _DayOfWeekLookup = new CronField(DayOfWeekSpec, 0, 6);
         _Duration = Duration;
      }

      public void Dispose() {
         _MinuteLookup = null;
         _HourLookup = null;
         _DayLookup = null;
         _MonthLookup = null;
         _DayOfWeekLookup = null;
      }

      public override string ToString() {
         var sb = new StringBuilder(string.Format("cron {0} {1} {2} {3} {4}", _MinuteLookup.CronSpec, _HourLookup.CronSpec, _DayLookup.CronSpec, _MonthLookup.CronSpec, _DayOfWeekLookup.CronSpec));
         if (_Duration != TimeSpan.Zero) {
            sb.Append(" lasting ");
            sb.Append(ItineraryConvert.ToString(_Duration));
         }
         return sb.ToString();
      }

      public IEnumerable<TimedEvent> GetRange(DateTime RangeStart, DateTime RangeEnd) {
         yield break;
      }
   }

   public class CronField {
      private int _Min, _Max;
      private string _CronSpec;
      private bool[] _Lookup;
      private List<int> _PickList;
      private static Regex _RegexStep = new Regex("/(\\d+)$");
      private static Regex _RegexSingle = new Regex("^\\d+$");
      private static Regex _RegexRange = new Regex("^(\\d+)-(\\d+)$");
      private static Regex _RegexMinMaxRange = new Regex("^([<>])(\\d+)$");

      public CronField(string CronSpec, int Min, int Max) {
         _Min = Min;
         _Max = Max;
         _Lookup = new bool[Length];
         Array.Clear(_Lookup, 0, Length);

         _CronSpec = CronSpec;
         Parse(CronSpec);
      }

      public int Length {
         get { return _Max - _Min + 1; }
      }

      public IEnumerable<int> Range {
         get {
            for (int i = _Min; i <= _Max; i++) {
               yield return i;
            }
         }
      }

      public string CronSpec {
         get { return _CronSpec; }
      }

      public bool this[int Index] {
         get {
            if (Index < _Min) throw new ArgumentOutOfRangeException("Index argument is below the minimum");
            if (Index > _Max) throw new ArgumentOutOfRangeException("Index argument is greater than the maximum");
            int ArrayIndex = Index - _Min;
            return _Lookup[ArrayIndex];
         }
      }

      public IEnumerable<int> PickList {
         get { return _PickList; }
      }

      private void Set(int Index, bool Value) {
         int ArrayIndex = Index - _Min;
         _Lookup[ArrayIndex] = Value;
      }

      private void Parse(string Cron) {
      }
   }
}
