using System;
using System.Collections.Generic;
using System.Text;

namespace Expl.Itinerary {
   /// <summary>
   /// Fetch a schedule from an IItineraryStateProvider
   /// </summary>
   public class FetchSchedule : IPrimitiveSchedule {
      protected string _ScheduleName;
      protected IItinerary _Itinerary;
      protected IScheduleDefinition _ScheduleInfo;

      /// <summary>
      /// Default constructor
      /// </summary>
      public FetchSchedule()
         : this("Every minute", new MemoryItinerary()) { }

      /// <summary>
      /// Fetch schedule from Itinerary
      /// </summary>
      /// <param name="ScheduleName">Schedule name to fetch</param>
      /// <param name="Itinerary">Itinerary state</param>
      public FetchSchedule(string ScheduleName, IItinerary Itinerary) {
         _ScheduleName = ScheduleName;
         _Itinerary = Itinerary;
         _ScheduleInfo = null;
      }

      public void Dispose() {
         _ScheduleName = null;
         _Itinerary = null;
      }

      public override string ToString() {
         return string.Format("@{{{0}}}", _ScheduleName);
      }

      public IScheduleDefinition ScheduleInfo {
         get {
            if (_ScheduleInfo == null) {
               _ScheduleInfo = _Itinerary.Schedules[_ScheduleName];
            }
            return _ScheduleInfo;
         }
      }

      public IEnumerable<TimedEvent> GetRange(DateTime RangeStart, DateTime RangeEnd) {
         yield break;
      }
   }
}
