using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using Antlr.Runtime;

namespace Expl.Itinerary {
   /// <summary>
   /// Helper class for calling parser to generate an ISchedule object from TDL
   /// </summary>
   public static class TDLParser {
      public static ISchedule Parse(string Text, IItinerary Itinerary) {
         TimeDefParser parser = GetParser(Text);
         parser.Itinerary = Itinerary;

         try {
            TimeDefParser.prog_return prog_ret = parser.prog();
            return prog_ret == null ? null : prog_ret.value;
         }
         catch (Exception ex) {
            throw new ParserException(ex.Message, ex);
         }
      }

      public static bool IsValid(string Text, IItinerary Itinerary) {
         try {
            ISchedule s = TDLParser.Parse(Text, Itinerary);
            return s is ISchedule;
         }
         catch (RecognitionException) { }
         return false;
      }

      public static DateTime ParseDateTime(string Text) {
         try {
            var ret = GetParser(Text).datetime_p();
            return ret.value;
         }
         catch (Exception ex) {
            throw new ParserException(ex.Message, ex);
         }
      }

      public static bool IsValidDateTime(string Text) {
         try {
            var dt = TDLParser.ParseDateTime(Text);
         }
         catch (RecognitionException) {
            return false;
         }
         return true;
      }

      public static TimeSpan ParseTimeSpan(string Text) {
         try {
            var ret = GetParser(Text).timespan_p();
            return ret.value;
         }
         catch (Exception ex) {
            throw new ParserException(ex.Message, ex);
         }
      }

      public static bool IsValidTimeSpan(string Text) {
         try {
            var ts = TDLParser.ParseTimeSpan(Text);
         }
         catch (RecognitionException) {
            return false;
         }
         return true;
      }

      private static TimeDefParser GetParser(string Text) {
         ICharStream input = new ANTLRStringStream(Text);
         TimeDefLexer lex = new TimeDefLexer(input);
         CommonTokenStream tokens = new CommonTokenStream(lex);
         return new TimeDefParser(tokens);
      }

      public class ParserException : Exception {
         public ParserException(string Message) : base(Message) { }
         public ParserException(string Message, Exception InnerException) : base(Message, InnerException) { }
      }
   }
}
