grammar TimeDef;

// Quick ref on set math: http://www.sonsothunder.com/devres/revolution/tips/arry002.htm

options {
   language=CSharp2;
   output=AST;
}
/*
@lexer::header {
   using Debug = System.Diagnostics.Debug;
}

@lexer::members {
   public IToken nextToken() {
      while (true) {
         this.token = null;
         this.channel = Token.DEFAULT_CHANNEL;
         this.tokenStartCharIndex = input.Index();
         this.tokenStartCharPositionInLine = input.CharPositionInLine;
         this.tokenStartLine = input.Line;
         this.text = null;
         if (input.LA(1) == '\n') return Token.EOF_TOKEN;
         Debug.WriteLine(string.Format("input.LA(1) = '{0}'", input.LA(1)));
         mTokens();
         if (this.token == null)
            Emit();
         else if (this.token == Token.SKIP_TOKEN)
            continue;
         return this.token;
      }
   }
}
*/
@parser::header {
   using System.Collections.Generic;
   using Debug = System.Diagnostics.Debug;
   using Expl.Itinerary;
}

@parser::members {
   private IItinerary _Itinerary = null;
   public IItinerary Itinerary {
      get { return _Itinerary; }
      set { _Itinerary = value; }
   }
}

@rulecatch {
   catch (RecognitionException) { throw; }
}

prog returns [ISchedule value]: (
   A=expr EOF
) {
   $value = $A.value;
};

//
// Expression rules
//
// Expression List
// Simple array list of expressions
expr returns [ISchedule value]
@init {
   List<ISchedule> Schedules = new List<ISchedule>();
}
: (
   A=boolnonintersection_p {
      //$value = $A.value;
      Schedules.Add($A.value);
   }
   (',' B=boolnonintersection_p
      {
         //ListSchedule o = $value as ListSchedule;
         //if (o == null) {
         //   $value = o = new ListSchedule();
         //   o.Schedules.Add($A.value);
         //}
         //o.Schedules.Add($B.value);
         Schedules.Add($B.value);
      }
   )*
) { $value = Schedules.Count > 1 ? new ListSchedule(Schedules) : Schedules[0]; };

atom returns [ISchedule value]:
   once_expr=once_p { $value = $once_expr.value; } |
   every_expr=every_p { $value = $every_expr.value; } |
   cron_expr=cron_p { $value = $cron_expr.value; } |
   fetch_expr=fetch_p { $value = $fetch_expr.value; } |
   void_expr=void_p { $value = $void_expr.value; } |
   paren_expr=paren_p { $value = $paren_expr.value; };


//
// Core expression primatives
//
once_p returns [OneTimeSchedule value]: (
   start=datetime_p (WS+ 'lasting' WS+ duration=timespan_p)?
) {
   $value = new OneTimeSchedule($start.value, duration==null ? TimeSpan.Zero : $duration.value);
};

every_p returns [IntervalSchedule value]: (
   'every' WS+ interval=timespan_p (WS+ 'since' WS+ sync=datetime_p)? (WS+ 'lasting' WS+ duration=timespan_p)?
) {
   $value = new IntervalSchedule($interval.value, duration==null ? TimeSpan.Zero : $duration.value, sync==null ? DateTime.MinValue : $sync.value);
};

cron_p returns [CronSchedule value]: (
   'cron' WS+
   minute=cron_field_p WS+
   hour=cron_field_p WS+
   day=cron_field_p WS+
   month=cron_field_p WS+
   dow=cron_field_p
   (WS+ 'lasting' WS+ duration=timespan_p)?
) {
   $value = new CronSchedule($minute.text, $hour.text, $day.text, $month.text, $dow.text, duration==null ? TimeSpan.Zero : $duration.value);
};

fetch_p returns [FetchSchedule value]: (
   '@{' fetch_name_p '}'
) {
   $value = new FetchSchedule($fetch_name_p.text, Itinerary);
};

void_p returns [VoidSchedule value]:
   'void' { $value = new VoidSchedule(); };

paren_p returns [ISchedule value]: '(' evt=expr { $value = $evt.value; } ')';


// Operator precedence:
// Intersection &
// Difference !&  (formerly ^)
// Union |
// Boolean Intersection &&
// Boolean Not Intersection !&&  (formerly ^^)
// List ,


//
// Expression computations
//

// Boolean NonIntersection
// Test Not (....[**)....]
// Test NonIntersection of two datetime spans
boolnonintersection_p returns [ISchedule value]: (
   WS* A=boolintersection_p { $value = $A.value; }
   (WS* '!&&' WS* B=boolintersection_p { $value = new BoolNonIntersectionSchedule($value, $B.value); } )?
   WS*
);

// Boolean Intersection
// Test (....[**)....]
// Like ANDing A and B
// Test intersection of two datetime spans
boolintersection_p returns [ISchedule value]: (
   A=union_p { $value = $A.value; }
   (WS* '&&' WS* B=union_p { $value = new BoolIntersectionSchedule($value, $B.value); } )?
);

// Union
// (****[**)****]
// Like ORing A and B
// Generate single datetime span, eliminate overlap redundancy
union_p returns [ISchedule value]
@init {
   List<ISchedule> Schedules = new List<ISchedule>();
}
: (
   A=difference_p {
      //$value = $A.value;
      Schedules.Add($A.value);
   }
   (WS* '|' WS* B=difference_p
      {
         //UnionSchedule o = $value as UnionSchedule;
         //if (o == null) {
         //   $value = o = new UnionSchedule();
         //   o.Schedules.Add($A.value);
         //}
         //o.Schedules.Add($B.value);
         Schedules.Add($B.value);
      }
   )*
) { $value = Schedules.Count > 1 ? new UnionSchedule(Schedules) : Schedules[0]; };

// Difference
// (****[..)****]
// Generate difference of two datetime spans
difference_p returns [ISchedule value]
@init {
   List<ISchedule> Schedules = new List<ISchedule>();
}
: (
   //A=intersection_p { $value = $A.value; }
   //('^' B=intersection_p { $value = new DifferenceSchedule($value, $B.value); } )?
   A=intersection_p {
      //$value = $A.value;
      Schedules.Add($A.value);
   }
   (WS* '!&' WS* B=intersection_p
      {
         //DifferenceSchedule o = $value as DifferenceSchedule;
         //if (o == null) {
         //   $value = o = new DifferenceSchedule();
         //   o.Schedules.Add($A.value);
         //}
         //o.Schedules.Add($B.value);
         Schedules.Add($B.value);
      }
   )*
) { $value = Schedules.Count > 1 ? new DifferenceSchedule(Schedules) : Schedules[0]; };

// Intersection
// (....[**)....]
// Like ANDing A and B
// Generate single datetime span
intersection_p returns [ISchedule value]: (
   A=atom { $value = $A.value; }
   (WS* '&' WS* B=atom { $value = new IntersectionSchedule($value, $B.value); } )?
);


//
// Time units
//
datetime_p returns [DateTime value]: (
   y=year_p '-' mo=month_p '-' d=day_p (WS+ h=hour24_p ':' m=minute60_p (':' s=second60_p ('.' ms=millisecond1000_p)?)?)? |
   h=hour24_p ':' m=minute60_p (':' s=second60_p ('.' ms=millisecond1000_p)?)?
) {
   $value = new DateTime(
      y==null ? DateTime.UtcNow.Year : $y.value,
      mo==null ? DateTime.UtcNow.Month : $mo.value,
      d==null ? DateTime.UtcNow.Day : $d.value,
      h==null ? 0 : $h.value,
      m==null ? 0 : $m.value,
      s==null ? 0 : $s.value,
      ms==null ? 0 : $ms.value);
};

year_p returns [int value]: UINT { $value = int.Parse($UINT.text); };
month_p returns [int value]: UINT { $value = int.Parse($UINT.text); };
day_p returns [int value]: UINT { $value = int.Parse($UINT.text); };
hour24_p returns [int value]: UINT { $value = int.Parse($UINT.text); };
minute60_p returns [int value]: UINT { $value = int.Parse($UINT.text); };
second60_p returns [int value]: UINT { $value = int.Parse($UINT.text); };
millisecond1000_p returns [int value]: UINT { $value = int.Parse($UINT.text); };

timespan_p returns [TimeSpan value]: (
   'T' (((d=days_p '.')? h=hours_p ':')? m=minutes_p ':')? s=seconds_p ('.' ms=milliseconds_p)?
) {
   $value = new TimeSpan(
      d==null ? 0 : $d.value,
      h==null ? 0 : $h.value,
      m==null ? 0 : $m.value,
      $s.value,
      ms==null ? 0 : $ms.value);
};

days_p returns [int value]: int_p { $value = int.Parse($int_p.text); };
hours_p returns [int value]: int_p { $value = int.Parse($int_p.text); };
minutes_p returns [int value]: int_p { $value = int.Parse($int_p.text); };
seconds_p returns [int value]: int_p { $value = int.Parse($int_p.text); };
milliseconds_p returns [int value]: int_p { $value = int.Parse($int_p.text); };

cron_field_p: cron_term_p (',' cron_term_p)*;
cron_term_p: '!'? (UINT | '/' | '-' | '*' | '>' | '<')+;

fetch_name_p: ALPHA (ALPHA|UINT|WS)*;

//
// Primitives
//
int_p: '-'? UINT;

UINT: ('0'..'9')+;
ALPHA: ('A'..'Z'|'a'..'z')+;
WS: (' '|'\t'|'\r\n'|'\r')+;
