using System;
using System.Collections.Generic;
using System.Text;
using Expl.Auxiliary;

namespace Expl.Itinerary {

   [AttributeUsage(AttributeTargets.Class, AllowMultiple=false, Inherited=false)]
   public class ParametersTypeName : Attribute {
      private string _TypeName = string.Empty;

      public ParametersTypeName(string TypeName) {
         _TypeName = TypeName;
      }

      public string TypeName {
         get { return _TypeName; }
      }
   }

   public interface ITask {
      /// <summary>
      /// Task name
      /// </summary>
      string Name { get; }

      /// <summary>
      /// Execute the task.
      /// </summary>
      /// <returns>TaskResult object containing status and return values</returns>
      TaskResult Execute(TimedEvent Event, ITaskParameters Parameters);
   }

   /// <summary>
   /// Parameters entity object
   /// Must be implemented with [Serializable] and all containing members must also be serializable
   /// </summary>
   public interface ITaskParameters { }

   public enum TaskStatusLevel {
      OK, Warning, Error
   }

   public class TaskResult {
      private TaskStatusLevel _status;
      private Dictionary<string, object> _extra;

      public TaskResult() {
         _status = TaskStatusLevel.Error;
         _extra = new Dictionary<string, object>();
      }

      public TaskResult(TaskStatusLevel Status) {
         _status = Status;
         _extra = new Dictionary<string, object>();
      }

      public TaskStatusLevel Status {
         get { return _status; }
         set { _status = value; }
      }

      public Dictionary<string, object> Extra {
         get { return _extra; }
      }

      public void DebugDump() {
         StringBuilder sb = new StringBuilder();
         sb.Append("Status: " + _status.ToString());
         foreach (string k in _extra.Keys) {
            sb.Append(", " + k + "=" + _extra[k].ToString());
         }
         ECDiagnostics.DebugText(sb.ToString());
      }
   }
}
